@@ -116,7 +116,7 @@ MODULE_PARM_DESC(osrun_time, "how many seconds to wait for the ICS2115 OS");
 
 #define DPRINT(cond, ...) \
        if ((dev->debug & (cond)) == (cond)) { \
-	     pr_debug(__VA_ARGS__); \
+	     snd_printk (__VA_ARGS__); \
        }
 #else
 #define DPRINT(cond, args...)
@@ -341,7 +341,7 @@ snd_wavefront_cmd (snd_wavefront_t *dev,
 
 	wfcmd = wavefront_get_command(cmd);
 	if (!wfcmd) {
-		dev_err(dev->card->dev, "command 0x%x not supported.\n",
+		snd_printk ("command 0x%x not supported.\n",
 			cmd);
 		return 1;
 	}
@@ -623,7 +623,7 @@ wavefront_get_sample_status (snd_wavefront_t *dev, int assume_rom)
 	/* check sample status */
     
 	if (snd_wavefront_cmd (dev, WFC_GET_NSAMPLES, rbuf, wbuf)) {
-		dev_err(dev->card->dev, "cannot request sample count.\n");
+		snd_printk ("cannot request sample count.\n");
 		return -1;
 	} 
     
@@ -635,8 +635,8 @@ wavefront_get_sample_status (snd_wavefront_t *dev, int assume_rom)
 		wbuf[1] = i >> 7;
 
 		if (snd_wavefront_cmd (dev, WFC_IDENTIFY_SAMPLE_TYPE, rbuf, wbuf)) {
-			dev_warn(dev->card->dev,
-				 "cannot identify sample type of slot %d\n", i);
+			snd_printk(KERN_WARNING "cannot identify sample "
+				   "type of slot %d\n", i);
 			dev->sample_status[i] = WF_ST_EMPTY;
 			continue;
 		}
@@ -661,9 +661,9 @@ wavefront_get_sample_status (snd_wavefront_t *dev, int assume_rom)
 			break;
 
 		default:
-			dev_err(dev->card->dev,
-				"unknown sample type for slot %d (0x%x)\n",
-				i, rbuf[0]);
+			snd_printk ("unknown sample type for "
+				    "slot %d (0x%x)\n", 
+				    i, rbuf[0]);
 		}
 
 		if (rbuf[0] != WF_ST_EMPTY) {
@@ -671,10 +671,9 @@ wavefront_get_sample_status (snd_wavefront_t *dev, int assume_rom)
 		} 
 	}
 
-	dev_info(dev->card->dev,
-		 "%d samples used (%d real, %d aliases, %d multi), %d empty\n",
-		 dev->samples_used, sc_real, sc_alias, sc_multi,
-		 WF_MAX_SAMPLE - dev->samples_used);
+	snd_printk ("%d samples used (%d real, %d aliases, %d multi), "
+		    "%d empty\n", dev->samples_used, sc_real, sc_alias, sc_multi,
+		    WF_MAX_SAMPLE - dev->samples_used);
 
 
 	return (0);
@@ -707,8 +706,8 @@ wavefront_get_patch_status (snd_wavefront_t *dev)
 		} else if (x == 3) { /* Bad patch number */
 			dev->patch_status[i] = 0;
 		} else {
-			dev_err(dev->card->dev,
-				"upload patch error 0x%x\n", x);
+			snd_printk ("upload patch "
+				    "error 0x%x\n", x);
 			dev->patch_status[i] = 0;
 			return 1;
 		}
@@ -725,8 +724,7 @@ wavefront_get_patch_status (snd_wavefront_t *dev)
 		}
 	
 	}
-	dev_info(dev->card->dev, "%d patch slots filled, %d in use\n",
-		 cnt, cnt2);
+	snd_printk ("%d patch slots filled, %d in use\n", cnt, cnt2);
 
 	return (0);
 }
@@ -762,8 +760,8 @@ wavefront_get_program_status (snd_wavefront_t *dev)
 		} else if (x == 1) { /* Bad program number */
 			dev->prog_status[i] = 0;
 		} else {
-			dev_err(dev->card->dev,
-				"upload program error 0x%x\n", x);
+			snd_printk ("upload program "
+				    "error 0x%x\n", x);
 			dev->prog_status[i] = 0;
 		}
 	}
@@ -774,7 +772,7 @@ wavefront_get_program_status (snd_wavefront_t *dev)
 		}
 	}
 
-	dev_info(dev->card->dev, "%d programs slots in use\n", cnt);
+	snd_printk ("%d programs slots in use\n", cnt);
 
 	return (0);
 }
@@ -798,7 +796,7 @@ wavefront_send_patch (snd_wavefront_t *dev, wavefront_patch_info *header)
 	munge_buf ((unsigned char *)&header->hdr.p, bptr, WF_PATCH_BYTES);
     
 	if (snd_wavefront_cmd (dev, WFC_DOWNLOAD_PATCH, NULL, buf)) {
-		dev_err(dev->card->dev, "download patch failed\n");
+		snd_printk ("download patch failed\n");
 		return -EIO;
 	}
 
@@ -839,7 +837,7 @@ wavefront_send_program (snd_wavefront_t *dev, wavefront_patch_info *header)
 	munge_buf ((unsigned char *)&header->hdr.pr, &buf[1], WF_PROGRAM_BYTES);
     
 	if (snd_wavefront_cmd (dev, WFC_DOWNLOAD_PROGRAM, NULL, buf)) {
-		dev_err(dev->card->dev, "download patch failed\n");
+		snd_printk ("download patch failed\n");	
 		return -EIO;
 	}
 
@@ -853,7 +851,7 @@ wavefront_freemem (snd_wavefront_t *dev)
 	char rbuf[8];
 
 	if (snd_wavefront_cmd (dev, WFC_REPORT_FREE_MEMORY, rbuf, NULL)) {
-		dev_err(dev->card->dev, "can't get memory stats.\n");
+		snd_printk ("can't get memory stats.\n");
 		return -1;
 	} else {
 		return demunge_int32 (rbuf, 4);
@@ -903,7 +901,7 @@ wavefront_send_sample (snd_wavefront_t *dev,
 		x = wavefront_find_free_sample(dev);
 		if (x < 0)
 			return -ENOMEM;
-		dev_info(dev->card->dev, "unspecified sample => %d\n", x);
+		snd_printk ("unspecified sample => %d\n", x);
 		header->number = x;
 	}
 
@@ -937,9 +935,9 @@ wavefront_send_sample (snd_wavefront_t *dev,
 
 		if (dev->rom_samples_rdonly) {
 			if (dev->sample_status[header->number] & WF_SLOT_ROM) {
-				dev_err(dev->card->dev,
-					"sample slot %d write protected\n",
-					header->number);
+				snd_printk ("sample slot %d "
+					    "write protected\n",
+					    header->number);
 				return -EACCES;
 			}
 		}
@@ -950,10 +948,10 @@ wavefront_send_sample (snd_wavefront_t *dev,
 	if (header->size) {
 		dev->freemem = wavefront_freemem (dev);
 
-		if (dev->freemem < 0 || dev->freemem < header->size) {
-			dev_err(dev->card->dev,
-				"insufficient memory to load %u byte sample.\n",
-				header->size);
+		if (dev->freemem < (int)header->size) {
+			snd_printk ("insufficient memory to "
+				    "load %d byte sample.\n",
+				    header->size);
 			return -ENOMEM;
 		}
 	
@@ -962,8 +960,8 @@ wavefront_send_sample (snd_wavefront_t *dev,
 	skip = WF_GET_CHANNEL(&header->hdr.s);
 
 	if (skip > 0 && header->hdr.s.SampleResolution != LINEAR_16BIT) {
-		dev_err(dev->card->dev,
-			"channel selection only possible on 16-bit samples");
+		snd_printk ("channel selection only "
+			    "possible on 16-bit samples");
 		return -EINVAL;
 	}
 
@@ -1059,8 +1057,8 @@ wavefront_send_sample (snd_wavefront_t *dev,
 			   header->size ?
 			   WFC_DOWNLOAD_SAMPLE : WFC_DOWNLOAD_SAMPLE_HEADER,
 			   NULL, sample_hdr)) {
-		dev_err(dev->card->dev, "sample %sdownload refused.\n",
-			header->size ? "" : "header ");
+		snd_printk ("sample %sdownload refused.\n",
+			    header->size ? "" : "header ");
 		return -EIO;
 	}
 
@@ -1085,8 +1083,8 @@ wavefront_send_sample (snd_wavefront_t *dev,
 		}
 
 		if (snd_wavefront_cmd (dev, WFC_DOWNLOAD_BLOCK, NULL, NULL)) {
-			dev_err(dev->card->dev,
-				"download block request refused.\n");
+			snd_printk ("download block "
+				    "request refused.\n");
 			return -EIO;
 		}
 
@@ -1147,13 +1145,13 @@ wavefront_send_sample (snd_wavefront_t *dev,
 		dma_ack = wavefront_read(dev);
 		if (dma_ack != WF_DMA_ACK) {
 			if (dma_ack == -1) {
-				dev_err(dev->card->dev,
-					"upload sample DMA ack timeout\n");
+				snd_printk ("upload sample "
+					    "DMA ack timeout\n");
 				return -EIO;
 			} else {
-				dev_err(dev->card->dev,
-					"upload sample DMA ack error 0x%x\n",
-					dma_ack);
+				snd_printk ("upload sample "
+					    "DMA ack error 0x%x\n",
+					    dma_ack);
 				return -EIO;
 			}
 		}
@@ -1197,7 +1195,7 @@ wavefront_send_alias (snd_wavefront_t *dev, wavefront_patch_info *header)
 	munge_int32 (*(&header->hdr.a.FrequencyBias+1), &alias_hdr[23], 2);
 
 	if (snd_wavefront_cmd (dev, WFC_DOWNLOAD_SAMPLE_ALIAS, NULL, alias_hdr)) {
-		dev_err(dev->card->dev, "download alias failed.\n");
+		snd_printk ("download alias failed.\n");
 		return -EIO;
 	}
 
@@ -1250,7 +1248,7 @@ wavefront_send_multisample (snd_wavefront_t *dev, wavefront_patch_info *header)
 	if (snd_wavefront_cmd (dev, WFC_DOWNLOAD_MULTISAMPLE, 
 			   (unsigned char *) (long) ((num_samples*2)+3),
 			   msample_hdr)) {
-		dev_err(dev->card->dev, "download of multisample failed.\n");
+		snd_printk ("download of multisample failed.\n");
 		kfree(msample_hdr);
 		return -EIO;
 	}
@@ -1273,7 +1271,7 @@ wavefront_fetch_multisample (snd_wavefront_t *dev,
 	munge_int32 (header->number, number, 2);
     
 	if (snd_wavefront_cmd (dev, WFC_UPLOAD_MULTISAMPLE, log_ns, number)) {
-		dev_err(dev->card->dev, "upload multisample failed.\n");
+		snd_printk ("upload multisample failed.\n");
 		return -EIO;
 	}
     
@@ -1292,16 +1290,16 @@ wavefront_fetch_multisample (snd_wavefront_t *dev,
 	
 		val = wavefront_read(dev);
 		if (val == -1) {
-			dev_err(dev->card->dev,
-				"upload multisample failed during sample loop.\n");
+			snd_printk ("upload multisample failed "
+				    "during sample loop.\n");
 			return -EIO;
 		}
 		d[0] = val;
 
 		val = wavefront_read(dev);
 		if (val == -1) {
-			dev_err(dev->card->dev,
-				"upload multisample failed during sample loop.\n");
+			snd_printk ("upload multisample failed "
+				    "during sample loop.\n");
 			return -EIO;
 		}
 		d[1] = val;
@@ -1336,7 +1334,7 @@ wavefront_send_drum (snd_wavefront_t *dev, wavefront_patch_info *header)
 	}
 
 	if (snd_wavefront_cmd (dev, WFC_DOWNLOAD_EDRUM_PROGRAM, NULL, drumbuf)) {
-		dev_err(dev->card->dev, "download drum failed.\n");
+		snd_printk ("download drum failed.\n");
 		return -EIO;
 	}
 
@@ -1354,7 +1352,7 @@ wavefront_find_free_sample (snd_wavefront_t *dev)
 			return i;
 		}
 	}
-	dev_err(dev->card->dev, "no free sample slots!\n");
+	snd_printk ("no free sample slots!\n");
 	return -1;
 }
 
@@ -1370,7 +1368,7 @@ wavefront_find_free_patch (snd_wavefront_t *dev)
 			return i;
 		}
 	}
-	dev_err(dev->card->dev, "no free patch slots!\n");
+	snd_printk ("no free patch slots!\n");
 	return -1;
 }
 #endif
@@ -1387,7 +1385,7 @@ wavefront_load_patch (snd_wavefront_t *dev, const char __user *addr)
 
 	if (copy_from_user (header, addr, sizeof(wavefront_patch_info) -
 			    sizeof(wavefront_any))) {
-		dev_err(dev->card->dev, "bad address for load patch.\n");
+		snd_printk ("bad address for load patch.\n");
 		err = -EFAULT;
 		goto __error;
 	}
@@ -1465,8 +1463,8 @@ wavefront_load_patch (snd_wavefront_t *dev, const char __user *addr)
 		break;
 
 	default:
-		dev_err(dev->card->dev, "unknown patch type %d.\n",
-			header->subkey);
+		snd_printk ("unknown patch type %d.\n",
+			    header->subkey);
 		err = -EINVAL;
 		break;
 	}
@@ -1529,13 +1527,13 @@ wavefront_synth_control (snd_wavefront_card_t *acard,
 	switch (wc->cmd) {
 		
 	case WFC_DISABLE_INTERRUPTS:
-		dev_dbg(dev->card->dev, "interrupts disabled.\n");
+		snd_printk ("interrupts disabled.\n");
 		outb (0x80|0x20, dev->control_port);
 		dev->interrupts_are_midi = 1;
 		return 0;
 
 	case WFC_ENABLE_INTERRUPTS:
-		dev_dbg(dev->card->dev, "interrupts enabled.\n");
+		snd_printk ("interrupts enabled.\n");
 		outb (0x80|0x40|0x20, dev->control_port);
 		dev->interrupts_are_midi = 1;
 		return 0;
@@ -1552,7 +1550,7 @@ wavefront_synth_control (snd_wavefront_card_t *acard,
 	case WFC_IDENTIFY_SLOT_TYPE:
 		i = wc->wbuf[0] | (wc->wbuf[1] << 7);
 		if (i <0 || i >= WF_MAX_SAMPLE) {
-			dev_err(dev->card->dev, "invalid slot ID %d\n",
+			snd_printk ("invalid slot ID %d\n",
 				i);
 			wc->status = EINVAL;
 			return -EINVAL;
@@ -1563,7 +1561,7 @@ wavefront_synth_control (snd_wavefront_card_t *acard,
 
 	case WFC_DEBUG_DRIVER:
 		dev->debug = wc->wbuf[0];
-		dev_dbg(dev->card->dev, "debug = 0x%x\n", dev->debug);
+		snd_printk ("debug = 0x%x\n", dev->debug);
 		return 0;
 
 	case WFC_UPLOAD_PATCH:
@@ -1580,8 +1578,8 @@ wavefront_synth_control (snd_wavefront_card_t *acard,
 		return 0;
 
 	case WFC_UPLOAD_SAMPLE_ALIAS:
-		dev_err(dev->card->dev,
-			"support for sample alias upload being considered.\n");
+		snd_printk ("support for sample alias upload "
+			"being considered.\n");
 		wc->status = EINVAL;
 		return -EINVAL;
 	}
@@ -1622,8 +1620,9 @@ wavefront_synth_control (snd_wavefront_card_t *acard,
 			break;
 
 		case WFC_UPLOAD_SAMPLE_ALIAS:
-			dev_err(dev->card->dev,
-				"support for sample aliases still being considered.\n");
+			snd_printk ("support for "
+				    "sample aliases still "
+				    "being considered.\n");
 			break;
 
 		case WFC_VMIDI_OFF:
@@ -1741,10 +1740,10 @@ snd_wavefront_internal_interrupt (snd_wavefront_card_t *card)
 		return;
 	}
 
-	scoped_guard(spinlock, &dev->irq_lock) {
-		dev->irq_ok = 1;
-		dev->irq_cnt++;
-	}
+	spin_lock(&dev->irq_lock);
+	dev->irq_ok = 1;
+	dev->irq_cnt++;
+	spin_unlock(&dev->irq_lock);
 	wake_up(&dev->interrupt_sleeper);
 }
 
@@ -1761,7 +1760,7 @@ snd_wavefront_internal_interrupt (snd_wavefront_card_t *card)
 */
 
 static int
-snd_wavefront_interrupt_bits(snd_wavefront_t *dev, int irq)
+snd_wavefront_interrupt_bits (int irq)
 
 {
 	int bits;
@@ -1781,7 +1780,7 @@ snd_wavefront_interrupt_bits(snd_wavefront_t *dev, int irq)
 		break;
 	
 	default:
-		dev_err(dev->card->dev, "invalid IRQ %d\n", irq);
+		snd_printk ("invalid IRQ %d\n", irq);
 		bits = -1;
 	}
 
@@ -1796,11 +1795,11 @@ wavefront_should_cause_interrupt (snd_wavefront_t *dev,
 	wait_queue_entry_t wait;
 
 	init_waitqueue_entry(&wait, current);
-	scoped_guard(spinlock_irq, &dev->irq_lock) {
-		add_wait_queue(&dev->interrupt_sleeper, &wait);
-		dev->irq_ok = 0;
-		outb(val, port);
-	}
+	spin_lock_irq(&dev->irq_lock);
+	add_wait_queue(&dev->interrupt_sleeper, &wait);
+	dev->irq_ok = 0;
+	outb (val,port);
+	spin_unlock_irq(&dev->irq_lock);
 	while (!dev->irq_ok && time_before(jiffies, timeout)) {
 		schedule_timeout_uninterruptible(1);
 		barrier();
@@ -1816,7 +1815,7 @@ wavefront_reset_to_cleanliness (snd_wavefront_t *dev)
 
 	/* IRQ already checked */
 
-	bits = snd_wavefront_interrupt_bits(dev, dev->irq);
+	bits = snd_wavefront_interrupt_bits (dev->irq);
 
 	/* try reset of port */
 
@@ -1886,7 +1885,7 @@ wavefront_reset_to_cleanliness (snd_wavefront_t *dev)
 	 */
 
 	if (!dev->irq_ok) {
-		dev_err(dev->card->dev, "intr not received after h/w un-reset.\n");
+		snd_printk ("intr not received after h/w un-reset.\n");
 		goto gone_bad;
 	} 
 
@@ -1910,18 +1909,18 @@ wavefront_reset_to_cleanliness (snd_wavefront_t *dev)
 					 dev->data_port, ramcheck_time*HZ);
 
 	if (!dev->irq_ok) {
-		dev_err(dev->card->dev, "post-RAM-check interrupt not received.\n");
+		snd_printk ("post-RAM-check interrupt not received.\n");
 		goto gone_bad;
 	} 
 
 	if (!wavefront_wait (dev, STAT_CAN_READ)) {
-		dev_err(dev->card->dev, "no response to HW version cmd.\n");
+		snd_printk ("no response to HW version cmd.\n");
 		goto gone_bad;
 	}
 	
 	hwv[0] = wavefront_read(dev);
 	if (hwv[0] == -1) {
-		dev_err(dev->card->dev, "board not responding correctly.\n");
+		snd_printk ("board not responding correctly.\n");
 		goto gone_bad;
 	}
 
@@ -1933,11 +1932,11 @@ wavefront_reset_to_cleanliness (snd_wavefront_t *dev)
 		
 		hwv[0] = wavefront_read(dev);
 		if (hwv[0] == -1) {
-			dev_err(dev->card->dev,
-				"on-board RAM test failed (bad error code).\n");
+			snd_printk ("on-board RAM test failed "
+				    "(bad error code).\n");
 		} else {
-			dev_err(dev->card->dev,
-				"on-board RAM test failed (error code: 0x%x).\n",
+			snd_printk ("on-board RAM test failed "
+				    "(error code: 0x%x).\n",
 				hwv[0]);
 		}
 		goto gone_bad;
@@ -1947,12 +1946,12 @@ wavefront_reset_to_cleanliness (snd_wavefront_t *dev)
 
 	hwv[1] = wavefront_read(dev);
 	if (hwv[1] == -1) {
-		dev_err(dev->card->dev, "incorrect h/w response.\n");
+		snd_printk ("incorrect h/w response.\n");
 		goto gone_bad;
 	}
 
-	dev_info(dev->card->dev, "hardware version %d.%d\n",
-		 hwv[0], hwv[1]);
+	snd_printk ("hardware version %d.%d\n",
+		    hwv[0], hwv[1]);
 
 	return 0;
 
@@ -1972,7 +1971,7 @@ wavefront_download_firmware (snd_wavefront_t *dev, char *path)
 
 	err = reject_firmware(&firmware, path, dev->card->dev);
 	if (err < 0) {
-		dev_err(dev->card->dev, "firmware (%s) download failed!!!\n", path);
+		snd_printk(KERN_ERR "firmware (%s) download failed!!!\n", path);
 		return 1;
 	}
 
@@ -1983,16 +1982,16 @@ wavefront_download_firmware (snd_wavefront_t *dev, char *path)
 		if (section_length == 0)
 			break;
 		if (section_length < 0 || section_length > WF_SECTION_MAX) {
-			dev_err(dev->card->dev,
-				"invalid firmware section length %d\n",
-				section_length);
+			snd_printk(KERN_ERR
+				   "invalid firmware section length %d\n",
+				   section_length);
 			goto failure;
 		}
 		buf++;
 		len++;
 
 		if (firmware->size < len + section_length) {
-			dev_err(dev->card->dev, "firmware section read error.\n");
+			snd_printk(KERN_ERR "firmware section read error.\n");
 			goto failure;
 		}
 
@@ -2009,14 +2008,15 @@ wavefront_download_firmware (snd_wavefront_t *dev, char *path)
 	
 		/* get ACK */
 		if (!wavefront_wait(dev, STAT_CAN_READ)) {
-			dev_err(dev->card->dev, "time out for firmware ACK.\n");
+			snd_printk(KERN_ERR "time out for firmware ACK.\n");
 			goto failure;
 		}
 		err = inb(dev->data_port);
 		if (err != WF_ACK) {
-			dev_err(dev->card->dev,
-				"download of section #%d not acknowledged, ack = 0x%x\n",
-				section_cnt_downloaded + 1, err);
+			snd_printk(KERN_ERR
+				   "download of section #%d not "
+				   "acknowledged, ack = 0x%x\n",
+				   section_cnt_downloaded + 1, err);
 			goto failure;
 		}
 
@@ -2028,7 +2028,7 @@ wavefront_download_firmware (snd_wavefront_t *dev, char *path)
 
  failure:
 	release_firmware(firmware);
-	dev_err(dev->card->dev, "firmware download failed!!!\n");
+	snd_printk(KERN_ERR "firmware download failed!!!\n");
 	return 1;
 }
 
@@ -2040,7 +2040,7 @@ wavefront_do_reset (snd_wavefront_t *dev)
 	char voices[1];
 
 	if (wavefront_reset_to_cleanliness (dev)) {
-		dev_err(dev->card->dev, "hw reset failed.\n");
+		snd_printk ("hw reset failed.\n");
 		goto gone_bad;
 	}
 
@@ -2064,7 +2064,7 @@ wavefront_do_reset (snd_wavefront_t *dev)
 						  (osrun_time*HZ));
 
 		if (!dev->irq_ok) {
-			dev_err(dev->card->dev, "no post-OS interrupt.\n");
+			snd_printk ("no post-OS interrupt.\n");
 			goto gone_bad;
 		}
 		
@@ -2074,7 +2074,7 @@ wavefront_do_reset (snd_wavefront_t *dev)
 						  dev->data_port, (10*HZ));
 		
 		if (!dev->irq_ok) {
-			dev_err(dev->card->dev, "no post-OS interrupt(2).\n");
+			snd_printk ("no post-OS interrupt(2).\n");
 			goto gone_bad;
 		}
 
@@ -2094,20 +2094,20 @@ wavefront_do_reset (snd_wavefront_t *dev)
 	if (dev->freemem < 0)
 		goto gone_bad;
 		
-	dev_info(dev->card->dev, "available DRAM %dk\n", dev->freemem / 1024);
+	snd_printk ("available DRAM %dk\n", dev->freemem / 1024);
 
 	if (wavefront_write (dev, 0xf0) ||
 	    wavefront_write (dev, 1) ||
 	    (wavefront_read (dev) < 0)) {
 		dev->debug = 0;
-		dev_err(dev->card->dev, "MPU emulation mode not set.\n");
+		snd_printk ("MPU emulation mode not set.\n");
 		goto gone_bad;
 	}
 
 	voices[0] = 32;
 
 	if (snd_wavefront_cmd (dev, WFC_SET_NVOICES, NULL, voices)) {
-		dev_err(dev->card->dev, "cannot set number of voices to 32.\n");
+		snd_printk ("cannot set number of voices to 32.\n");
 		goto gone_bad;
 	}
 
@@ -2187,8 +2187,8 @@ snd_wavefront_detect (snd_wavefront_card_t *card)
 		dev->fw_version[0] = rbuf[0];
 		dev->fw_version[1] = rbuf[1];
 
-		dev_info(dev->card->dev, "firmware %d.%d already loaded.\n",
-			 rbuf[0], rbuf[1]);
+		snd_printk ("firmware %d.%d already loaded.\n",
+			    rbuf[0], rbuf[1]);
 
 		/* check that a command actually works */
       
@@ -2197,24 +2197,22 @@ snd_wavefront_detect (snd_wavefront_card_t *card)
 			dev->hw_version[0] = rbuf[0];
 			dev->hw_version[1] = rbuf[1];
 		} else {
-			dev_err(dev->card->dev,
-				"not raw, but no hardware version!\n");
+			snd_printk ("not raw, but no "
+				    "hardware version!\n");
 			return -1;
 		}
 
 		if (!wf_raw) {
 			return 0;
 		} else {
-			dev_info(dev->card->dev,
-				 "reloading firmware as you requested.\n");
+			snd_printk ("reloading firmware as you requested.\n");
 			dev->israw = 1;
 		}
 
 	} else {
 
 		dev->israw = 1;
-		dev_info(dev->card->dev,
-			 "no response to firmware probe, assume raw.\n");
+		snd_printk ("no response to firmware probe, assume raw.\n");
 
 	}
 
